#include "WebServer.h"

#include <QEvent>
#include <QSocketNotifier>

#include <QTcpServer>
#include <QTcpSocket>

static const char* GetEventName(QEvent::Type type)
{
	switch(type)
	{
	case QEvent::MetaCall: return "MetaCall";
	case QEvent::SockAct: return "SockAct";
	case QEvent::DeferredDelete: return "DeferredDelete";
	case QEvent::ChildAdded: return "ChildAdded";
	case QEvent::ChildRemoved: return "ChildRemoved";
	default:
		return "";
	}
	return "";
}

WebServer::WebServer(const QHostAddress &host, quint16 port)
{
	m_host = host;
	m_port = port;
	m_count = 0;

    m_pServer = new QTcpServer();
    connect(m_pServer, SIGNAL(newConnection()), this, SLOT(acceptConnection()));
}

WebServer::~WebServer()
{

}

void WebServer::start()
{
	m_pServer->listen(m_host, m_port);
}

void WebServer::acceptConnection()
{
	qDebug("[WebServer] ---------------------->");
	qDebug("[WebServer] newConnection received");

	QTcpSocket* pSocket = m_pServer->nextPendingConnection ();
	QObject::connect(pSocket, SIGNAL(readyRead()), this, SLOT(incomingData()));
    QObject::connect(pSocket, SIGNAL(disconnected()), this, SLOT(disconnected()));
    QObject::connect(pSocket, SIGNAL(destroyed()), this, SLOT(destroyed()));
}

void WebServer::incomingData()
{
	qDebug("[WebServer] readyRead received");

	QIODevice* pDevice = qobject_cast<QIODevice*>(sender());
	if (!pDevice) return;


	QString szResponse;
	m_count++;
	QString szContent = "Count is: " + QString::number(m_count);

	szResponse = "HTTP/1.1 200 OK\r\n"\
		"Content-Type: text/html; charset=utf-8\r\n"
		"Content-Length: " + QString::number(szContent.length())  + "\r\n"
		"Connection: close\r\n"
		"\r\n" + szContent;

	qDebug("[WebServer] socket content sent: %s", qPrintable(szContent));

	pDevice->write(szResponse.toAscii());
	pDevice->close();
}

void WebServer::disconnected()
{
	qDebug("[WebServer] socket disconnected");
	QIODevice* pDevice = qobject_cast<QIODevice*>(sender());
	if (!pDevice) return;

	pDevice->deleteLater();
}

void WebServer::destroyed()
{
	qDebug("[WebServer] socket destroyed");
	qDebug("[WebServer] <----------------------");
}

/*
void WebServer::incomingRequest(quint32 requestID,
                                const QHttpRequestHeader & header,
                                QxtWebContent *deviceContent)
{
    qDebug("[WebServer] (Request %d) Method=%s, URI: %s", requestID, qPrintable(header.method()), qPrintable(header.path()));
    QxtHttpSessionManager::incomingRequest(requestID, header, deviceContent);
    qDebug("[WebServer] (Request %d) Request processed", requestID);
}
*/

bool WebServer::eventFilter(QObject *obj, QEvent *event)
{
	Q_UNUSED(obj)
	qDebug("[WebServer] Event received %s(%d)", GetEventName(event->type()), event->type());

	switch(event->type())
	{
	case QEvent::SockAct:
		qDebug("[WebServer] Socket descriptor %d", ((QSocketNotifier*)event)->socket());
		break;
	default:
		break;
	}

	return false;
}

