//============================================================================
// Name        : srpv.cpp
// Author      : Eric Beuque
// Version     :
// Copyright   : Your copyright notice
// Description : Server for recording and processing videos, Ansi-style
//============================================================================

#include <iostream>
#include <QCoreApplication>
#include <QtGlobal>
#include <QDateTime>

#include "WebServer.h"
#include "WorkerThread.h"

#include <signal.h>
#include <unistd.h>

using namespace std;

void sig_term(int sig)
{
	qDebug("[Main] Received signal: %d", sig);
	QCoreApplication::quit();
}

void log_function(QtMsgType type, const char *msg)
{
	QDateTime dtNow = QDateTime::currentDateTime();
	fprintf(stderr, "[%s] %s\n", qPrintable(dtNow.toString()), qPrintable(msg));
}

int main(int argc, char *argv[])
{
	qInstallMsgHandler(log_function);

	// Initialize the QT application
	QCoreApplication a(argc, argv);
	
	signal(SIGTERM, sig_term);
	signal(SIGINT, sig_term);

	// Port
	quint16 port = 8081;

	qDebug("[Main] Initialization server");
	WebServer *pWebServer = new WebServer(QHostAddress::Any, port);
	a.installEventFilter (pWebServer);

	qDebug("[Main] Starting server");
    pWebServer->start();

	qDebug("[Main] Starting worker");
	WorkerThread* pThread = new WorkerThread();
	pThread->start();

	qDebug("[Main] Starting the main loop");
	a.exec();
	qDebug("[Main] Exit the main loop");

	pThread->abort();
	qDebug("[WorkerThread] Wait worker thread");
	pThread->wait();
	qDebug("[WorkerThread] Destroy worker thread");
	delete pThread;

	qDebug("[WorkerThread] Destroy service");
	delete pWebServer;
	
	return 0;
}

